"""
The generic form class.  Useful for 95% of form functionality.
"""
from serializer import getData
from config import FORM_TO_DB_MAP
from universal import DevError, uPrint, FormError
import importlib
import db
import java.lang.Exception
# This is the script package where all your forms are located, you can change it if needed.
BASE_MODULE = "forms."

class Form():
	form_payload = None
	initial_data = None
	prefix = None
	form_name = None
	db_table = None
	idx = None
	userId = None
	module_name = None
	result = None #result of insert
	event = None
	def __init__(self, form_name, container=None, data=None, event=None, prefix="_out_", userId=None):
		"""
		Usually we will pass the container right in that has access to the form data elements.  But say we want to read from a CSV file
		then we would pass in each row via the data keyword instead.  Even keyword is there for closing the vision window/perspective window upon success
		"""
		self.form_name = validateFormName(form_name)
		self.db_table = validateDBTable(form_name)
		self.module_name = validateModule(form_name)
		self.form_payload = getData(container, prefix=prefix)
		self.idx = self.form_payload['idx']
		# Because we don't to insert the idx or update it, its a separate parameter
		del self.form_payload['idx']
		if userId is None:
			self.userId = self.form_payload['userId']
		else:
			self.userId = userId
		self.event = event
		
	def insert(self):
		i = importlib.import_module(self.module_name)

		successfullyValidated=True
		try:
			i.validateNew(self.form_payload)
		except AttributeError:
			print "no validate new function for %s"%(self.module_name)
			pass
		except universal.FormError, e:
			print "detected form error"
			successfullyValidated=False
			system.gui.warningBox(str(e))

		if successfullyValidated:
			successfullyModified=True
			try:
				i.modifyNew(self.form_payload)
			except AttributeError:
				print "no modifyNew new function for %s"%(self.module_name)
				pass
			except universal.FormError, e:
				print "detected form error"
				successfullyModified=False
				system.gui.warningBox(str(e))
		
			if successfullyModified:
				print "validated and modifed, about to insert"
				try:
					print "about to insert"
					self.result, self.idx = db.query.insert(self.form_payload, self.db_table, getKey = 1)
					print "inserted"
				except java.lang.Exception, e:
					self.result = False
					if "Duplicate entry" in str(e.cause):
						violatedColumn =  str(str(e.cause).split('key')[1].strip()[1:-1])
						value = self.form_payload[violatedColumn]
						system.gui.warningBox("We already have a record with %s in the %s field."%(value, violatedColumn))
				###
				# Here - we do universal displaying of success or failure
				###
				if self.idx != -1:
					try:
						i.postinsert(self.form_payload, self.idx, self.result)
					except AttributeError:
						print "no postInsert function for %s"%(self.module_name)
						pass
					except universal.FormError, e:
						system.gui.warningBox(str(e))
					print "COMPLETED INSERT!"
					system.gui.messageBox("Created record %i"%(self.idx))
	
					if not self.result:
						system.gui.messageBox("Difference detected between what was saved and what is in database")
	
					if self.event is not None:
						# Here we would universally shut down popup window, but for now just vision
						system.nav.closeParentWindow(self.event)
				
def validateFormName(form_name):
	"""Checks that form name exists in configuration mapping"""
	try:
		FORM_TO_DB_MAP[form_name]
		return form_name
	except KeyError:
		raise DevError("Form %s does not exist in your configuration.  Please add it"%(form_name))

def validateDBTable(form_name):
	"""Checks that form databsae table exists"""
	try:
		return FORM_TO_DB_MAP[form_name]['dbTable']
	except KeyError:
		raise DevError("Form %s exists, but lacks database property. Please add 'dbTable' to your configuration"%(form_name))

def validateModule(form_name):
	"""Checks that module listed exists or not"""
	try:
		module_name = FORM_TO_DB_MAP[form_name]['module']
	except KeyError:
		raise DevError("Form %s exists, but lacks database property. Please add 'dbTable' to your configuration"%(form_name))
	try:
		module_name = BASE_MODULE + module_name
		importlib.import_module(module_name)
		return module_name
	except ImportError:
		raise DevError("Please make a script %s or fix your configuration to point to correct module."%(module_name))