"""
Query builder + checker of query insert/update, all in one.
"""
import system.db
import db.util
from universal import uPrint
import java.lang.Exception
import time
import system.dataset
import java.sql.Timestamp

def insert(dictionary, tableName, getKey = 0, sleep=1):
	"""
	For insertions
	Args:
		data: dictionary of key (column) value (value) pairs
		tableName: str, what table are we inserting into?
		getKey: int, 0 or 1, do you want the id back?
		sleep: int - how long to wait beofre checking
	"""
	columnInfo = db.util.getDBColumnInfo(tableName)
	dictKeys = dictionary.keys()
	uPrint("in insert function")
	if db.util.dictonaryTableCheckForInsert(columnInfo, dictKeys):
		columnNames = ', '.join([x for x in dictKeys])
		values = [x for x in dictionary.values()]
		questionMarks = ','.join(['?' for x in range(len(dictKeys))])
		realQuery = "INSERT INTO %s(%s) VALUES (%s)"%(tableName, columnNames, questionMarks)
		uPrint("q: " + str(realQuery))
		uPrint("v: " + str(values))
		newIdx = system.db.runPrepUpdate(realQuery, values, getKey = 1)
		result = check(dictionary, tableName, newIdx, sleep=sleep)
		
		if getKey == 1:
			return result,newIdx
		else:
			return result
		
def update(dictionary, tableName, idx=-1, sleep=1, whereClause=None):
	"""
	This function takes a dictionary with column:column-value pairs, a table name, and an insert query with it
	Args:
		dictionary: dict, key-value pairs are column name - column value
		tableName: name of the table to insert into
		idx: int, idx of the row to update
		sleep: float, default 1, how long in between rechecks,
		whereClaues: none, otherwise list of keys in dictionary that make up our where clause
	Returns:
		result - boolean of if update worked
	"""	
	if idx == -1 and whereClause is None:
		raise ValueError("You must provide either an idx or a where clause for update to work")
	
	columnInfo = db.util.getDBColumnInfo(tableName)
	dictKeys = dictionary.keys()
	uPrint("in update function")
	if db.util.dictonaryTableCheckForUpdate(columnInfo, dictKeys):
		middleOfQuery = ', '.join([x + ' = ?' for x in dictionary.keys()])
		if whereClause is None:
			realQuery = "UPDATE %s SET %s WHERE idx = ?"%(tableName, middleOfQuery)
			values = [x for x in dictionary.values()]
			values.append(idx)
		else:
			# I know this one liner might be frowned upon, but here's what it does
			# take all the keys that make up the whereClause column, and sets them equal to their value
			# like column1 = "someValue" AND column2 = "someOthervalue"
			values = [x for x in dictionary.values()]
			for item in whereClause:
				values.append(dictionary[item])
			whereClauseString = " AND ".join([str(key) +" = " + "?" for key in whereClause])
			realQuery = "UPDATE %s SET %s WHERE %s"%(tableName, middleOfQuery, whereClauseString)
			uPrint(values)
			uPrint("real update where cluase query:" + str(realQuery))
		system.db.runPrepUpdate(realQuery, values, getKey = 1)
		if idx !=-1:
			uPrint("checking based on idx")
			result = check(dictionary, tableName, idx, sleep=sleep)
			uPrint("result is %s"%(str(result)))
		else:
			# To do - make check workt with custom where clause as well, just not important atm
			result = True
		return result

def check(dictionary, tableName, idx, retry = 5, sleep=.5, count=0):
	"""
	This query is to be called after a update or insert query has run to verify that the information
	has been saved to the database appropriately.
	Args:
		dictionary: dict, {"columnInDBName": value}
		tableName: str, name of the table in database
		idx: int, idx of the record in tableName
		visibilityProperty: a reference to self.custom or similar type abstract property, use showLoading as the convention
	Returns:
		boolean, false if things didn't update after 5 attemps, true if they did
	ACTIONS: Turns off loading client tag
	"""
	uPrint("count: " + str(count))
	if count >= retry:
		return False
		
	columnNames = ', '.join([x for x in dictionary.keys()])
	selectQuery = "SELECT %s FROM %s WHERE idx = %i"%(columnNames, tableName, idx)
	results = system.dataset.toPyDataSet(system.db.runQuery(selectQuery))
	badResult = False
#	dictionary['quotationName'] = 'wrongName'
	
	for key in dictionary.keys():
		databaseValue = results[0][str(key)]
		dictionaryValue = dictionary[key]
		dbValueCheck = isinstance(databaseValue,java.sql.Timestamp)
		# Convert dbvalue if need be
		if dbValueCheck is True:
			databaseValue = system.date.fromMillis(databaseValue.time)

		if databaseValue != dictionaryValue:
			uPrint("mismatched values for %s"%(str(key)))
			uPrint("dbValue: " + str(databaseValue))
			uPrint("dictValue: " + str(dictionaryValue))
			uPrint("dbType: " + str(type(databaseValue)))
			uPrint("dictType: " + str(type(dictionaryValue)))

			if type(databaseValue) is float and type(dictionaryValue) is float:
				if abs(databaseValue - dictionaryValue) > .0001:
					badResult = True
					break
			else:
				badResult = True
				break
		
	if badResult:
		uPrint("bad result, retrying in %s second"%(str(sleep)))
		time.sleep(sleep)
		count+=1
		check(dictionary, tableName, idx, count=count)
	else:
		return True