"""
Improved database functions with checker and query builder.
"""
import system.db
import java.lang.Exception
from universal import uPrint


def getDBColumnInfo(tableName):
	"""
	Fetchs the columns from the database and returns them as a list of strings
	Args:
		tableName: str, name of database table
	Returns:
		list of tuples of strings of column names of the form
		("column name", "type", "nulls allowed?")
	"""
	try:
		ds = system.db.runQuery("DESCRIBE %s"%(tableName))
		l = []
		for row in ds:
			if row[0] != 'idx':
				# As idx is not something you ever insert, its' the database id convention we use
				l.append((row[0], row[1], row[2]))
		uPrint("columns: " + str(l))
		return l
	except java.lang.Exception, e:
		raise ValueError(e.cause)

def dictionaryKeysAllMatchColumn(columnInfo, dictionaryKeys):
	"""
	This function takes in a list of tuples with info about the columns from getDBColumnInfo
	and compares them to dictionary keys to make sure that each dictionary key aligns with a column.
	Args:
		columnInfo: list of tuples, provided via getDBColumnInfo
		dictionaryKeys: dictionary.keys() from a dictionary
	Returns:
		True if every key in the dictionary matches up with a column in a table
		False otherwise
	"""
	# This turns the first part of each tuple of the list into its own list, so just a list of column names
	columnNames = [x[0] for x in columnInfo]
	for item in dictionaryKeys:
		if item not in columnNames:
			return False
	return True
	
def necessaryColumnsFilledIn(columnInfo, dictionaryKeys):
	"""
	This function takes in a list of tuples with info about the colums (from getDBColumnInfo)
	and compares them to the dcitonary keys, checking that the columsn that CANNOT be null are included in the
	dictonary at a minimum.
	Args:
		columnInfo: list of tuples, provided via getDBColumnInfo
		dictionaryKeys: dictionary.keys() from a dictionary
	Returns:
		True if every key in the dictionary matches up with a column in a table
		False otherwise
	"""
	necessaryColumns = [x[0] for x in columnInfo if x[2] == 'NO']
	missingColumns = []
	for column in necessaryColumns:
		if column not in dictionaryKeys:
			missingColumns.append(column)
			uPrint(' ****column not in dictionary****' + str(column))
	if len(missingColumns) > 0:
		raise ValueError("You are missing %s necessary columns in your dictionary."%(str(', '.join(missingColumns))))

def dictonaryTableCheckForUpdate(columnInfo, dictionaryKeys):
	"""
	Wrapper function around dictionaryKeysAllMatchColumn and
	necessaryColumnsFilledIn to make sure boht conditions are met as a minimum.
	Args:
	
	Returns:
		True if everything matches and non-nullable fields are met
		Error otherwise with a helpful message of some sort	
	"""
	allMatched = dictionaryKeysAllMatchColumn(columnInfo, dictionaryKeys)
	if not allMatched:
		mismatchedColumns = []
		columnNames = [x[0] for x in columnInfo]
		for item in dictionaryKeys:
			if item not in columnNames:
				mismatchedColumns.append(item)
		raise ValueError('%s columns in dictionary do not exist in database.'%(', '.join(mismatchedColumns)))
	return True

def dictonaryTableCheckForInsert(columnInfo, dictionaryKeys):
	"""
	Wrapper function around dictionaryKeysAllMatchColumn and
	necessaryColumnsFilledIn to make sure boht conditions are met as a minimum.
	Args:
	
	Returns:
		True if everything matches and non-nullable fields are met
		Error otherwise with a helpful message of some sort	
	"""
	allMatched = dictionaryKeysAllMatchColumn(columnInfo, dictionaryKeys)
	if not allMatched:
		mismatchedColumns = []
		columnNames = [x[0] for x in columnInfo]
		for item in dictionaryKeys:
			if item not in columnNames:
				mismatchedColumns.append(item)
		raise ValueError('%s columns in dictionary do not exist in database.'%(', '.join(mismatchedColumns)))
	necessaryColumnsFilledIn(columnInfo, dictionaryKeys)
	return True