"""
NOTE:  TO USE THIS MODULE, YOUR WINDOW MUST USE THE FOLLOW NAMIng CONVENTIONS

Component names use "_out_" to denote the beginning of the naming convention.  So you can put whatever text before "_out_" that you want, or you ca begin with "_out_"
which is a good idea to do on root containers, so you know exactly what you are grabbing from the screen.

After the name of 
"""

"""
THIS IS THE VISION SCRIPTS FOR THE COMPONENT NAMING CONVENTION
"""
def parseProperties(prop, prefix="_out_"):
	"""
	Helper function for createDictionaryFromComponents
	Args:
		prop: a property on a container
	Returns:
		tuple, (column, value) - to be used in dictionary
	"""
	propName = str(prop)
	if prefix in str(prop):
		column  = propName.split(prefix)[1]
		value = prop.getValue()
		return (column, value)
	else:
		return (None, None)
	
def parseComponent(component, prefix="_out_"):
	"""
	Helper function for createDictionaryFromComponents
	Args:
		component: a component, or a proeprty on a container
	Returns:
		tuple, (column, value) - to be used to add to the dictonary
	"""
	try:
		componentName = str(component.name)
		if prefix in componentName:
#			print "goign to save component %s to db"%(componentName)
			columnAndType  = componentName.split(prefix)[1]
			column = '_'.join(columnAndType.split("_")[:-1])
			dataType = columnAndType.split("_")[-1].lower()
#			print "column name in db: " + str(column)
#			print "property to get off component: " + str(dataType)
			if dataType == 'text':
				value = component.text
			elif dataType == 'float':
				value = component.floatValue
			elif dataType == 'floatround':
				value = round(component.floatValue, 2)
			elif dataType == 'selectedvalue':
				value = component.selectedValue
			elif dataType == 'selected':
				value = component.selected
			elif dataType == 'integer':
				value = component.intValue
			elif dataType == 'date':
				value = component.date
			elif dataType == 'data':
				value = component.data
			else:
				value = None
				for i in component.properties:
#					print str(i)
					if str(i).lower() == dataType.lower():
						value = component.getPropertyValue(str(i))
						break
			return (column, value)
		else:
			return (None, None)
	except Exception, e:
		print "fe: " + str(e)

def getDataVision(container, prefix="_out_"):
	"""
	The final step in automation of the CRUD cycle.  How to use this -
	To make sure your component is set properly in the dictionary to be send to the database
	it MUST follow this format.
	OthertextAboutComponenthere_out_dbcolumnname_propertyToGrab
	So for example for a text field whos text property I want to save to a column "name" in the db,
	textfield_out_name_text
	Properties we can handle:
		_text
		_selectedValue
		_float
		_int
		_date
	And we can also handle custom properties, which you would just have to give the full name of
	textfield_out_name_customPropertyNameHere
	Args:
		container: can be used on any container, but doesn't do a deep search
		into child containers of containers (yet).  Most likely a Root Container,
		or a Form Container.
	Returns:
		dictionary: keys are column names of database, values are values to be updated/inserted
	"""
	import system.gui
	
	ds = {}
	# iterate through root container customer properties first, often things to take out of there
	for prop in container.properties:
		columnName, value = parseProperties(prop, prefix=prefix)
		if columnName is not None and value is not None:
			ds[columnName] = value
		
	componentsInContainer = container.components
	
	while len(componentsInContainer) != 0:
		currentComponent = componentsInContainer.pop()
		try:
			columnName, value = parseComponent(currentComponent, prefix=prefix)
		except Exception, e:
			print "e: " + str(e)
		if columnName is not None and value is None:
			print "Error getting value for column %s"%(columnName)
			ds[columnName] = None
		elif columnName is None and value is None:
			pass
		elif columnName is not None and value is not None:
			ds[columnName] = value
	return ds

"""
THIS IS THE PERSPECTIVE SCRIPST FOR THE NAMING CONVETNION
"""

"""
For keeping track of GUI functions that require a whole view.
"""

def recursiveGetAllComponents(component):
	"""
	Gets all child components from any component, usually a container.  Shallow, not deep.
	Args:
		any component. usually a container.
	Returns:
		[component1, component2, ...]
	"""
	return [child.name for child in component.children]

def getContainerComponents(root, container, dbTableName, idx):
	"""
	Use this to the dictonary of data objects you will use for the rest of these functions.
	Args:
		root: root
		container: str, name of container, or maybe view will work here too? maybe
		dbTableName: str, name of database table to query the good column names from in order to populate the data
		idx: int, idx of the record in the database
	Returns:
		dict:	{
					'fullcomponentname': 	{
												type: either 'dd', 'tf', 'nf', 'cb', etc
												dbColumn: str, name of related column in database
												prop: prop field you care about for inserting values/checking
												dbValue: retrieved value from the database
											}
				}
	"""
	import system.db
	import database.util
	# Get list of all components that require db info
	allComponents = recursiveGetAllComponents(root.getChild(container))
	dataComponents = [x for x in allComponents if '_out_' in x]
	system.perspective.print("good data: " + str(dataComponents))
	dataDict = {}
	columns = []
	for component in dataComponents:
		componentType, dbColumn, propField = parseComponentName(component)
		columns.append(dbColumn)
		dataDict[component] = {'type':componentType, 'dbColumn':dbColumn, 'prop':propField, 'dbValue':None}
	system.perspective.print("dataDict 2: " + str(dataDict))
	# Build query and fetch info
	query = "SELECT %s FROM %s WHERE idx = %i"%(', '.join(columns), dbTableName, idx)
	system.perspective.print(query)
	queryResult = database.util.rowToDictionary(system.db.runQuery(query))
	system.perspective.print(queryResult)
	# Update dictionary so now it matches the end result
	for k, v in dataDict.items():
		if v['dbColumn'] in queryResult.keys():
			v['dbValue'] = queryResult[v['dbColumn']]
	system.perspective.print("dataDict 2: " + str(dataDict))
	# Update the gui
	for k, v in dataDict.items():
		setValue(k, v, root, container)
	return dataDict

def parseComponentName(componentName):
	"""
	Takes a component that is made with our naming convention and parses it out
	Args:
		componentName: string, name
	Returns:
		tuple: (componentType, dbColumn, propField) 
	"""
	component = componentName.split('_')
	componentType = component[0]
	dbColumn = component[2]
	propField = component[3]
	return (componentType, dbColumn, propField)

	
"""
THIS IS WHAT YOU ACTUALLY CALL
"""

def getData(container, prefix="_out_"):
	"""
	Use this to get all the form data on a view or vision window returned to you as a dictionary.
	Args;
		container: either a root container (vision) or view (perspective), we use this to determine what to call
		prefix: str, what prefix do we use to signify which components we want
	Returns:
		dictionary: of all the component data from the window/view, based on naming conventions
		that can be found at the top of this module
	"""
	from universal import uPrint
	if "com.inductiveautomation.factorypmi.application.components.BasicContainer" in str(type(container)):
		uPrint("vision serializer")
		data = getDataVision(container, prefix=prefix)
		uPrint("Data: " + str(data))
		return data
	elif "com.inductiveautomation.perspective.gateway.script.CompontModelScriptWrapper$SafetyWrapper" in str(type(container)):
		uPrint("perspective")
